//**************************************************************************
// ﾌｧｲﾙ内容     「gpt_pwm.ino」モータドライブ基板へPWM出力でサーボ、モータを制御
// Copyright    ジャパンマイコンカーラリー実行委員会
// ライセンス   This software is released under the MIT License.
//              http://opensource.org/licenses/mit-license.php
//**************************************************************************

// インクルード宣言
#include "mcr_gpt_lib.h"

// 定数の宣言
#define PWM_CYCLE     47999             //  16ms/{1/(HOCO/16)} = 0.016 /{1/(48e6/16)} = 48000
#define SERVO_CENTER  4500              // 1.5ms/{1/(HOCO/16)} = 0.0015/{1/(48e6/16)} =  4500
#define HANDLE_STEP   26                // １度あたりの値

// プロトタイプ宣言
void motor(int accele_l, int accele_r);
void handle(int angle);
unsigned char dipsw_get(void);

void setup() {
  pinMode( 25, INPUT );                 // DIPSW1
  pinMode( 26, INPUT );                 // DIPSW2

  setGPTterminal( 4, 6 );               // P406(GTIOC1B):サーボPWM
  setGPTterminal( 4, 5 );               // P405(GTIOC1A):左モータPWMs
  setGPTterminal( 4, 3 );               // P403(GTIOC3A):右モータPWM

  GTIOC1A = 0;
  GTIOC1B = SERVO_CENTER;
  startPWM_GPT1( GTIOCA | GTIOCB , DIV16, PWM_CYCLE );  // GPT1 GTIOCAとBを使用 分周:16 周期:PWM_CYCLE
  
  GTIOC3A = 0;
  startPWM_GPT3( GTIOCA          , DIV16, PWM_CYCLE );  // GPT3 GTIOCAを使用    分周:16 周期:PWM_CYCLE

  // CN5 モータドライブ基板
  //pinMode( 27 , OUTPUT );         // LED2 rev.1.0bは入力端子なのでLEDは点灯しない
  //pinMode( 28 , OUTPUT );         // LED3 rev.1.0bは入力端子なのでLEDは点灯しない
  //pinMode( 29 , OUTPUT );         // サーボPWMなので設定しない
  //pinMode( 30 , OUTPUT );         // 右モータPWMなので設定しない
  pinMode(31, OUTPUT);              // 右モータ方向
  //pinMode( 32 , OUTPUT );         // 左モータPWMなので設定しない
  pinMode(33, OUTPUT);              // 左モータ方向
  pinMode(34, INPUT);               // プッシュスイッチ
}

void loop() {

  handle( 0 );
  motor( 0, 0 );
  delay( 1000 );

  handle(10 );
  motor( 0, 25 );
  delay( 1000 );

  handle( -10 );
  motor( 25, 0 );
  delay( 1000 );

  handle( 20 );
  motor( 0, -50 );
  delay( 1000 );

  handle( -20 );
  motor( -50, 0 );
  delay( 1000 );
}

//*******************************************************************
// モータ速度制御
// 引数　 左モータ:-100～100、右モータ:-100～100
//        0で停止、100で正転100%、-100で逆転100%
// 戻り値 なし
//*******************************************************************
void motor(int accele_l, int accele_r) {
  int sw_data;

  sw_data = dipsw_get() + 7;  // 7～10
  accele_l = accele_l * sw_data / 10;
  accele_r = accele_r * sw_data / 10;

  // 左モータ制御
  if(accele_l >= 0) {
    R_PORT4->PODR_b.PODR2 = 0;
    accele_l = (long)(PWM_CYCLE + 1) * accele_l / 100;
    GTIOC3A = accele_l;
  } else {
    R_PORT4->PODR_b.PODR2 = 1;
    accele_l = (long)(PWM_CYCLE + 1) * (-accele_l) / 100;
    GTIOC3A = accele_l;
  }

  // 右モータ制御
  if(accele_r >= 0) {
    R_PORT4->PODR_b.PODR4 = 0;
    accele_r = (long)(PWM_CYCLE + 1) * accele_r / 100;
    GTIOC1A = accele_r;
  } else {
    R_PORT4->PODR_b.PODR4 = 1;
    accele_r = (long)(PWM_CYCLE + 1) * (-accele_r) / 100;
    GTIOC1A = accele_r;
  }
}

//*******************************************************************
// サーボハンドル操作
// 引数　 サーボ操作角度：-90～90
//        -90で左へ90度、0でまっすぐ、90で右へ90度回転
//*******************************************************************
void handle(int angle) {
  // サーボが左右逆に動く場合は、「-」を「+」に替えてください
  GTIOC1B = SERVO_CENTER - angle * HANDLE_STEP;
}

//******************************************************************
// ディップスイッチ値読み込み
// 戻り値 スイッチ値 0～3
//******************************************************************
unsigned char dipsw_get(void) {
  unsigned char sw;

  sw = (!(R_PORT3->PIDR_b.PIDR7) << 1) + (!(R_PORT3->PIDR_b.PIDR6));

  return sw;
}