//**************************************************************************
// 対象マイコン Renesas Electronics RMC_RA4M1 Rev.1.0bボード(R7FA4M1AB3CFP9 100pin)
//              ※RMC_RA4M1 Rev.2.0ボードは、ポート配置が換わるためこのプログラムでは動作しません
// ﾌｧｲﾙ内容     Basicマイコンカー　サーボセンタ・サーボ角度　確認プログラム
// バージョン   Ver.1.00
// Date         2023.11.07
// Copyright    ジャパンマイコンカーラリー実行委員会
// ライセンス   This software is released under the MIT License.
//              http://opensource.org/licenses/mit-license.php
//**************************************************************************

//======================================
// インクルード
//======================================
#include "mcr_gpt_lib.h"

//======================================
// シンボル定義
//======================================

// モータ、サーボのPWM関係
#define PWM_CYCLE     47999             //  16ms/(1/HOCO*16) = 0.016 /(1/48e6*16) = 48000
#define SERVO_CENTER  4499              // 1.5ms/(1/HOCO*16) = 0.0015/(1/48e6*16) =  4500
#define HANDLE_STEP   26                // Handle step

// ピン名定義
#define LED_D3      (D24)
#define DIP_SW1     (D25)
#define DIP_SW2     (D26)

//======================================
// グローバル変数の宣言
//======================================
volatile int mode;                   // プログラムのモード
volatile int servo_offset;           // サーボオフセット
volatile int servo_angle;            // サーボ角度

//**********************************************************************
// setup
//**********************************************************************
void setup()
{
  servo_offset = SERVO_CENTER;

  setGPTterminal( 4, 6 );           // P406 GTIOC1B :サーボPWMとして使用
  setGPTterminal( 4, 5 );           // P405 GTIOC1A :左モータPWMとして使用
  setGPTterminal( 4, 3 );           // P403 GTIOC3A :右モータPWMとして使用

  GTIOC1A = 0;
  GTIOC1B = SERVO_CENTER;
  startPWM_GPT1( GTIOCA | GTIOCB , DIV16, PWM_CYCLE );  // GPT1 GTIOC1Aと1Bを使用 分周:16 周期:PWM_CYCLE

  GTIOC3A = 0;
  startPWM_GPT3( GTIOCA          , DIV16, PWM_CYCLE );  // GPT3 GTIOC3Aを使用     分周:16 周期:PWM_CYCLE

  // RMC-RA4M1基板上のLEDとDIPスイッチ
  pinMode( LED_D3, OUTPUT );
  pinMode( DIP_SW1, INPUT );
  pinMode( DIP_SW2, INPUT );

  // CN5 モータドライブ基板
  //pinMode( 27 , OUTPUT );         // LED2 rev.1.0bは入力端子なのでLEDは点灯しない
  //pinMode( 28 , OUTPUT );         // LED3 rev.1.0bは入力端子なのでLEDは点灯しない
  //pinMode( 29 , OUTPUT );         // サーボPWMなので設定しない
  //pinMode( 30 , OUTPUT );         // 右モータPWMなので設定しない
  pinMode( 31, OUTPUT );            // 右モータ方向
  //pinMode( 32 , OUTPUT );         // 左モータPWMなので設定しない
  pinMode( 33, OUTPUT );            // 左モータ方向
  pinMode( 34, INPUT_PULLUP);       // プッシュスイッチ

  // CN7 センサ基板
  pinMode( 35, INPUT );
  pinMode( 36, INPUT );
  pinMode( 37, INPUT );
  pinMode( 38, INPUT );
  pinMode( 39, INPUT );
  pinMode( 40, INPUT );
  pinMode( 41, INPUT );
  pinMode( 42, INPUT );

  Serial.begin( 9600 );             // シリアル通信 速度9600bps

  while ( !Serial ) {
    // シリアルが接続されるまで待つ
  }

  R_PORT3->PODR_b.PODR5  = 0; // LED_D3 点灯
}

//**********************************************************************
// loop
//**********************************************************************
void loop()
{
  switch( mode ) {
    case 0:
      Serial.print( "\r\n"
                    "\r\n"
                    "\r\n"
                    "サーボセンタ・サーボ角度　確認プログラム\r\n"
                    "\r\n"
                    "'1' : サーボセンタ調整\r\n"
                    "'2' : サーボ角度確認\r\n"
                    "\r\n"
      );
      mode = 1;
      break;

    case 1:
      if ( Serial.available() ) {       // 受信データがあるか？
        char c = Serial.read();         // 1文字読み込む
        if( c == '1' ) {
          Serial.print( "1\r\n" );
          mode = 11;
        } else if( c == '2' ) {
          Serial.print( "2\r\n" );
          servo_angle = 0;
          mode = 21;
        }
      }
      break;

    case 11:
      Serial.print( "\r\n"
                    "'サーボセンタ調整\r\n"
                    "\r\n"
                    "'Z'キー : サーボセンタ値 +1\r\n"
                    "'X'キー : サーボセンタ値 -1\r\n"
                    "\r\n"
                    "'A'キー : サーボセンタ値+10\r\n"
                    "'S'キー : サーボセンタ値-10\r\n"
                    "\r\n"
                    "Enterキー : メニューに戻ります\r\n"
                    "\r\n"
      );
      Serial.print( servo_offset );
      Serial.print( "  \r" );
      mode = 12;
      break;

    case 12:
      if ( Serial.available() ) {       // 受信データがあるか？
        char c = Serial.read();         // 1文字読み込む
        switch( c ) {
          case 'Z':
          case 'z':
            servo_offset++;
            if( servo_offset > 10000 ) {
              servo_offset = 10000;
            }
            Serial.print( servo_offset );
            Serial.print( "  \r" );
            break;

          case 'A':
          case 'a':
            servo_offset += 10;
            if( servo_offset > 10000 ) {
              servo_offset = 10000;
            }
            Serial.print( servo_offset );
            Serial.print( "  \r" );
            break;

          case 'X':
          case 'x':
            servo_offset--;
            if( servo_offset < 1000 ) {
              servo_offset = 1000;
            }
            Serial.print( servo_offset );
            Serial.print( "  \r" );
            break;

          case 'S':
          case 's':
            servo_offset -= 10;
            if( servo_offset < 1000 ) {
              servo_offset = 1000;
            }
            Serial.print( servo_offset );
            Serial.print( "  \r" );
            break;

          case 0x0d:  // エンター
            mode = 0;
            break;

          default:
            break;
        }
        GTIOC1B = servo_offset;
      }
      break;

    case 21:
      Serial.print( "\r\n"
                    "'サーボ角度確認\r\n"
                    "\r\n"
                    "'Z'キー : サーボ角度 +1\r\n"
                    "'X'キー : サーボ角度 -1\r\n"
                    "\r\n"
                    "'A'キー : サーボ角度+10\r\n"
                    "'S'キー : サーボ角度-10\r\n"
                    "\r\n"
                    "Enterキー : メニューに戻ります\r\n"
                    "\r\n"
      );
      Serial.print( servo_angle );
      Serial.print( "  \r" );
      mode = 22;
      break;

    case 22:
      if ( Serial.available() ) {       // 受信データがあるか？
        char c = Serial.read();         // 1文字読み込む
        switch( c ) {
          case 'Z':
          case 'z':
            servo_angle++;
            if( servo_angle > 90 ) {
              servo_angle = 90;
            }
            Serial.print( servo_angle );
            Serial.print( "  \r" );
            break;

          case 'A':
          case 'a':
            servo_angle += 10;
            if( servo_angle > 90 ) {
              servo_angle = 90;
            }
            Serial.print( servo_angle );
            Serial.print( "  \r" );
            break;

          case 'X':
          case 'x':
            servo_angle--;
            if( servo_angle < -90 ) {
              servo_angle = -90;
            }
            Serial.print( servo_angle );
            Serial.print( "  \r" );
            break;

          case 'S':
          case 's':
            servo_angle -= 10;
            if( servo_angle < -90 ) {
              servo_angle = -90;
            }
            Serial.print( servo_angle );
            Serial.print( "  \r" );
            break;

          case 0x0d:  // エンター
            mode = 0;
            break;

          default:
            break;
        }
        GTIOC1B = servo_offset + servo_angle * HANDLE_STEP;
      }
      break;
  }
}

//**********************************************************************
// end of file
//**********************************************************************

/*
改訂内容
Ver.1.00 2023.11.07 作成
*/
